@@ -56,8 +56,8 @@ class AnnotationResult():
         ----------
         by
             Column name of :attr:`~celltypist.classifier.AnnotationResult.predicted_labels` specifying the prediction type which the summary is based on.
-            Set to `majority_voting` if you want to summarize for the majority voting classifier.
-            (Default: `predicted_labels`)
+            Set to `'majority_voting'` if you want to summarize for the majority voting classifier.
+            (Default: `'predicted_labels'`)
 
         Returns
         ----------
@@ -69,7 +69,7 @@ class AnnotationResult():
         df.sort_values(['counts'], ascending=False, inplace=True)
         return df
 
-    def to_adata(self, insert_labels: bool = True, insert_conf: bool = True, insert_decision: bool = False, insert_prob: bool = False, prefix: str = '') -> AnnData:
+    def to_adata(self, insert_labels: bool = True, insert_conf: bool = True, insert_conf_by: str = 'predicted_labels', insert_decision: bool = False, insert_prob: bool = False, prefix: str = '') -> AnnData:
         """
         Insert the predicted labels, decision or probability matrix, and (if majority voting is done) majority voting results into the AnnData object.
 
@@ -79,8 +79,12 @@ class AnnotationResult():
             Whether to insert the predicted cell type labels and (if majority voting is done) majority voting-based labels into the AnnData object.
             (Default: `True`)
         insert_conf
-            Whether to insert the confidence scores (the maximal probability of query cells) into the AnnData object.
+            Whether to insert the confidence scores into the AnnData object.
             (Default: `True`)
+        insert_conf_by
+            Column name of :attr:`~celltypist.classifier.AnnotationResult.predicted_labels` specifying the prediction type which the confidence scores are based on.
+            Setting to `'majority_voting'` will insert the confidence scores corresponding to the majority-voting result.
+            (Default: `'predicted_labels'`)
         insert_decision
             Whether to insert the decision matrix into the AnnData object.
             (Default: `False`)
@@ -97,13 +101,20 @@ class AnnotationResult():
             1) **predicted_labels**, individual prediction outcome for each cell.
             2) **over_clustering**, over-clustering result for the cells.
             3) **majority_voting**, the cell type label assigned to each cell after the majority voting process.
-            4) **conf_score**, the confidence score (maximum probability across all cell types) of each cell.
+            4) **conf_score**, the confidence score of each cell.
             5) **name of each cell type**, which represents the decision scores (or probabilities if `insert_prob` is `True`) of a given cell type across cells.
         """
         if insert_labels:
             self.adata.obs[[f"{prefix}{x}" for x in self.predicted_labels.columns]] = self.predicted_labels
         if insert_conf:
-            self.adata.obs[f"{prefix}conf_score"] = self.probability_matrix.max(axis=1).values
+            if insert_conf_by == 'predicted_labels':
+                self.adata.obs[f"{prefix}conf_score"] = self.probability_matrix.max(axis=1).values
+            elif insert_conf_by == 'majority_voting':
+                if insert_conf_by not in self.predicted_labels:
+                    raise KeyError(f"🛑 Did not find the column `majority_voting` in the `AnnotationResult.predicted_labels`, perform majority voting beforehand or use `insert_conf_by = 'predicted_labels'` instead")
+                self.adata.obs[f"{prefix}conf_score"] = [row[self.predicted_labels.majority_voting[index]] for index, row in self.probability_matrix.iterrows()]
+            else:
+                raise KeyError(f"🛑 Unrecognized `insert_conf_by` value ('{insert_conf_by}'), should be one of `'predicted_labels'` or `'majority_voting'`")
         if insert_prob:
             self.adata.obs[[f"{prefix}{x}" for x in self.probability_matrix.columns]] = self.probability_matrix
         elif insert_decision:
@@ -124,7 +135,7 @@ class AnnotationResult():
             (Default: `False`)
         format
             Format of output figures. Default to vector PDF files (note dots are still drawn with png backend).
-            (Default: `pdf`)
+            (Default: `'pdf'`)
         prefix
             Prefix for the output figures. Default to no prefix used.
 
@@ -194,8 +205,13 @@ class AnnotationResult():
                 self.decision_matrix.to_excel(writer, sheet_name="Decision Matrix")
                 self.probability_matrix.to_excel(writer, sheet_name="Probability Matrix")
 
-    def __str__(self):
-        return f"{self.cell_count} cells predicted into {len(np.unique(self.predicted_labels['predicted_labels']))} cell types"
+    def __repr__(self):
+        base = f"CellTypist prediction result for {self.cell_count} query cells"
+        base += f"\n    predicted_labels: data frame with {self.predicted_labels.shape[1]} {'columns' if self.predicted_labels.shape[1] > 1 else 'column'} ({str(list(self.predicted_labels.columns))[1:-1]})"
+        base += f"\n    decision_matrix: data frame with {self.cell_count} query cells and {self.decision_matrix.shape[1]} cell types"
+        base += f"\n    probability_matrix: data frame with {self.cell_count} query cells and {self.probability_matrix.shape[1]} cell types"
+        base += f"\n    adata: AnnData object referred"
+        return base
 
 class Classifier():
     """
@@ -265,7 +281,7 @@ class Classifier():
             self.adata.var_names_make_unique()
             if not float(self.adata.X.max()).is_integer():
                 logger.warn(f"⚠️ Warning: the input file seems not a raw count matrix. The prediction result may be biased")
-            if (self.adata.n_vars >= 80000) or (len(self.adata.var_names[0]) >= 30) or (len(self.adata.obs_names.intersection(pd.Index(['GAPDH', 'ACTB', 'CALM1']))) >= 1):
+            if (self.adata.n_vars >= 80000) or (len(self.adata.var_names[0]) >= 30) or (len(self.adata.obs_names.intersection(pd.Index(['GAPDH', 'ACTB', 'CALM1', 'PTPRC']))) >= 1):
                 raise ValueError(f"🛑 The input matrix is detected to be a gene-by-cell matrix. Please provide a cell-by-gene matrix or add the input transpose option")
             sc.pp.normalize_total(self.adata, target_sum=1e4)
             sc.pp.log1p(self.adata)
@@ -276,13 +292,13 @@ class Classifier():
             self.adata = sc.read(filename) if isinstance(filename, str) else filename
             self.adata.var_names_make_unique()
             if self.adata.X.min() < 0:
-                logger.info("👀 Detected scaled expression in the input data, will try the .raw attribute")
+                logger.info("👀 Detected scaled expression in the input data, will try the `.raw` attribute")
                 try:
                     self.indata = self.adata.raw.X
                     self.indata_genes = self.adata.raw.var_names
                     self.indata_names = self.adata.raw.obs_names
                 except Exception as e:
-                    raise Exception(f"🛑 Fail to use the .raw attribute in the input object. {e}")
+                    raise Exception(f"🛑 Fail to use the `.raw` attribute in the input object. {e}")
             else:
                 self.indata = self.adata.X
                 self.indata_genes = self.adata.var_names
@@ -302,11 +318,11 @@ class Classifier():
         ----------
         mode
             The way cell prediction is performed.
-            For each query cell, the default (`best match`) is to choose the cell type with the largest score/probability as the final prediction.
-            Setting to `prob match` will enable a multi-label classification, which assigns 0 (i.e., unassigned), 1, or >=2 cell type labels to each query cell.
-            (Default: `best match`)
+            For each query cell, the default (`'best match'`) is to choose the cell type with the largest score/probability as the final prediction.
+            Setting to `'prob match'` will enable a multi-label classification, which assigns 0 (i.e., unassigned), 1, or >=2 cell type labels to each query cell.
+            (Default: `'best match'`)
         p_thres
-            Probability threshold for the multi-label classification. Ignored if `mode` is `best match`.
+            Probability threshold for the multi-label classification. Ignored if `mode` is `'best match'`.
             (Default: 0.5)
 
         Returns
@@ -320,7 +336,10 @@ class Classifier():
         """
         logger.info(f"🔗 Matching reference genes in the model")
         k_x = np.isin(self.indata_genes, self.model.classifier.features)
-        logger.info(f"🧬 {k_x.sum()} features used for prediction")
+        if k_x.sum() == 0:
+            raise ValueError(f"🛑 No features overlap with the model. Please provide gene symbols")
+        else:
+            logger.info(f"🧬 {k_x.sum()} features used for prediction")
         k_x_idx = np.where(k_x)[0]
         #self.indata = self.indata[:, k_x_idx]
         self.indata_genes = self.indata_genes[k_x_idx]
@@ -332,6 +351,7 @@ class Classifier():
         self.indata = (self.indata[:, k_x_idx] - means_) / sds_
         self.indata[self.indata > 10] = 10
 
+        ni, fs, cf = self.model.classifier.n_features_in_, self.model.classifier.features, self.model.classifier.coef_
         self.model.classifier.n_features_in_ = lr_idx.size
         self.model.classifier.features = self.model.classifier.features[lr_idx]
         self.model.classifier.coef_ = self.model.classifier.coef_[:, lr_idx]
@@ -340,6 +360,9 @@ class Classifier():
         decision_mat, prob_mat, lab = self.model.predict_labels_and_prob(self.indata, mode = mode, p_thres = p_thres)
         logger.info("✅ Prediction done!")
 
+        #restore model after prediction
+        self.model.classifier.n_features_in_, self.model.classifier.features, self.model.classifier.coef_ = ni, fs, cf
+
         cells = self.indata_names
         return AnnotationResult(pd.DataFrame(lab, columns=['predicted_labels'], index=cells, dtype='category'), pd.DataFrame(decision_mat, columns=self.model.classifier.classes_, index=cells), pd.DataFrame(prob_mat, columns=self.model.classifier.classes_, index=cells), self.adata)
 
@@ -374,7 +397,7 @@ class Classifier():
             A :class:`~pandas.Series` object showing the over-clustering result.
         """
         if 'connectivities' not in self.adata.obsp:
-            logger.info("👀 Can not detect a neighborhood graph, construct one before the over-clustering")
+            logger.info("👀 Can not detect a neighborhood graph, will construct one before the over-clustering")
             adata = self.adata.copy()
             self.adata.obsm['X_pca'], self.adata.obsp['connectivities'], self.adata.obsp['distances'], self.adata.uns['neighbors'] = Classifier._construct_neighbor_graph(adata)
         else:
@@ -394,9 +417,7 @@ class Classifier():
                 resolution = 30
         logger.info(f"⛓️ Over-clustering input data with resolution set to {resolution}")
         sc.tl.leiden(self.adata, resolution=resolution, key_added='over_clustering')
-        oc_column = self.adata.obs.over_clustering
-        self.adata.obs.drop(columns=['over_clustering'], inplace=True)
-        return oc_column
+        return self.adata.obs.pop('over_clustering')
 
     @staticmethod
     def majority_vote(predictions: AnnotationResult, over_clustering: Union[list, tuple, np.ndarray, pd.Series, pd.Index], min_prop: float = 0) -> AnnotationResult:
